"use strict";

exports.__esModule = true;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var _helpers2 = require("./helpers");

var _helpers3 = _interopRequireDefault(_helpers2);

exports["default"] = function (_ref2) {
  var Plugin = _ref2.Plugin;
  var t = _ref2.types;

  var visitor = {};

  var _helpers = _helpers3["default"](t);

  var buildChildren = _helpers.buildChildren;
  var extractOpenArguments = _helpers.extractOpenArguments;
  var flattenExpressions = _helpers.flattenExpressions;
  var toReference = _helpers.toReference;
  var toFunctionCall = _helpers.toFunctionCall;
  var toFunctionCallStatement = _helpers.toFunctionCallStatement;
  var attrsToAttrCalls = _helpers.attrsToAttrCalls;

  visitor.JSXOpeningElement = {
    exit: function exit(_ref3, parent, scope) {
      var name = _ref3.name;
      var attributes = _ref3.attributes;
      var selfClosing = _ref3.selfClosing;

      var tag = toReference(name);
      var args = [tag];

      var _extractOpenArguments = extractOpenArguments(attributes);

      var key = _extractOpenArguments.key;
      var statics = _extractOpenArguments.statics;
      var attrs = _extractOpenArguments.attrs;
      var hasSpread = _extractOpenArguments.hasSpread;

      var elementFunction = selfClosing ? "elementVoid" : "elementOpen";

      // Only push arguments if they're needed
      if (key || statics || attrs) {
        args.push(key || t.literal(null));
      }
      if (statics || attrs) {
        args.push(statics ? t.arrayExpression(statics) : t.literal(null));
      }

      // If there is a spread element, we need to use
      // the elementOpenStart/elementOpenEnd syntax.
      // This allows spreads to be transformed into
      // attr(name, value) calls.
      if (hasSpread) {
        attrs = attrs.map(attrsToAttrCalls(scope));

        return t.sequenceExpression([toFunctionCall("elementOpenStart", args)].concat(attrs, [toFunctionCall("elementOpenEnd", [tag])]));
      } else if (attrs) {
        for (var _iterator = attrs, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
          var _ref;

          if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref = _iterator[_i++];
          } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref = _i.value;
          }

          var _name = _ref[0];
          var value = _ref[1];

          args.push(_name, value);
        }
      }

      return toFunctionCallStatement(elementFunction, args);
    }
  };

  visitor.JSXClosingElement = {
    exit: function exit(_ref4) {
      var name = _ref4.name;

      return toFunctionCallStatement("elementClose", [toReference(name)]);
    }
  };

  visitor.JSXElement = {
    exit: function exit(_ref5) {
      var openingElement = _ref5.openingElement;
      var children = _ref5.children;
      var closingElement = _ref5.closingElement;

      // Filter out empty children, and transform JSX expressions
      // into normal expressions.
      children = buildChildren(children);

      var elements = [openingElement].concat(children);
      if (closingElement) {
        elements.push(closingElement);
      }

      // Turn all sequence expressions into function statements.
      return flattenExpressions(elements);
    }
  };

  return new Plugin("incremental-dom", { visitor: visitor });
};

module.exports = exports["default"];